CREATE OR REPLACE TRIGGER ARTISTS_TRIGGER
  BEFORE INSERT 
    OR UPDATE OF NAME, SURNAME, IS_GROUP, LOCALE
  ON ARTISTS
  FOR EACH ROW
DECLARE
  IS_SURNAME_FIRST CHAR(1);  
BEGIN
  DBMS_OUTPUT.PUT_LINE('ARTISTS_TRIGGER ...');

  /* Set default locale */
  IF :NEW.LOCALE IS NULL THEN
    :NEW.LOCALE := 'en';
  END IF;

  /* Group or individual? */
   IF :NEW.IS_GROUP='T' THEN

     DBMS_OUTPUT.PUT('group - ');

     /* A group cannot have a surname */
     IF :NEW.SURNAME IS NOT NULL THEN
       RAISE_APPLICATION_ERROR(-20501,
         'Group cannot have surname');
     ELSE
       SELECT PARSE_TITLE(:NEW.NAME, :NEW.LOCALE)
         INTO :NEW.SORT_NAME
         FROM DUAL;
     END IF;

   ELSE /* Individual */

     DBMS_OUTPUT.PUT('individual - ');

     /* if surname is normally first, dont use comma */
     SELECT SURNAME_FIRST
       INTO IS_SURNAME_FIRST
       FROM SURNAME_ORDER
       WHERE LOCALE=:NEW.LOCALE;

     IF IS_SURNAME_FIRST='T' THEN
       :NEW.SORT_NAME :=
         :NEW.SURNAME || ' ' || :NEW.NAME;
     ELSE
       :NEW.SORT_NAME :=
         :NEW.SURNAME || ', ' ||  :NEW.NAME;
      END IF;

  END IF;

  DBMS_OUTPUT.PUT_LINE(:NEW.SORT_NAME);

END;
/
