DROP TABLE CIRCLES;
DROP TABLE LINES;
DROP TABLE RECTANGLES;
DROP TABLE TRIANGLES;
DROP TYPE LINE_TYPE;
DROP TYPE RECTANGLE_TYPE;
DROP TYPE TRIANGLE_TYPE;

CREATE OR REPLACE TYPE POINT_TYPE AS OBJECT (
  X	NUMBER,
  Y	NUMBER
);
/

CREATE TABLE CIRCLES (
   RADIUS  NUMBER,
   CENTER  POINT_TYPE
);

CREATE TYPE LINE_TYPE AS OBJECT (
  START_POINT POINT_TYPE,
  END_POINT   POINT_TYPE,
  MEMBER FUNCTION GETLENGTH RETURN NUMBER
);
/

CREATE TYPE BODY LINE_TYPE AS

  MEMBER FUNCTION GETLENGTH RETURN NUMBER AS
    BEGIN
      RETURN SQRT(
          POWER(START_POINT.X-END_POINT.X,2)
         +POWER(START_POINT.Y-END_POINT.Y,2)
      );
    END;

END;
/

CREATE TABLE LINES OF LINE_TYPE;
INSERT INTO LINES VALUES(POINT_TYPE(1,1), POINT_TYPE(4,5));
COMMIT;

CREATE OR REPLACE TYPE RECTANGLE_TYPE AS OBJECT (
  TOP_LEFT POINT_TYPE,
  WIDTH    NUMBER,
  HEIGHT   NUMBER,
  MAP MEMBER FUNCTION GETAREA RETURN NUMBER,
  MEMBER FUNCTION CONTAINS (PT IN POINT_TYPE) RETURN NUMBER
)
/

CREATE TYPE BODY RECTANGLE_TYPE AS

    MAP MEMBER FUNCTION GETAREA RETURN NUMBER AS
           BEGIN
                   RETURN WIDTH*HEIGHT;
           END;

    MEMBER FUNCTION CONTAINS
     (PT IN POINT_TYPE) RETURN NUMBER AS
                   IS_INSIDE NUMBER := 0;
           BEGIN
                   IF(  PT.X > TOP_LEFT.X
                      AND PT.X < TOP_LEFT.X + WIDTH
                      AND PT.Y > TOP_LEFT.Y
                      AND PT.Y < TOP_LEFT.Y + HEIGHT
                   )
                   THEN
                      IS_INSIDE := 1;
                   END IF;
                   RETURN IS_INSIDE;
           END;
END;
/


CREATE TABLE RECTANGLES (
     LABEL VARCHAR2(25),
     RECTANGLE RECTANGLE_TYPE
);

INSERT INTO RECTANGLES VALUES('One',
  RECTANGLE_TYPE(POINT_TYPE(10,50), 40, 20));

INSERT INTO RECTANGLES VALUES('Two',
  RECTANGLE_TYPE(POINT_TYPE( 0, 0), 10, 10));

COMMIT;

CREATE TYPE TRIANGLE_TYPE AS OBJECT (
  PT_A POINT_TYPE,
  PT_B POINT_TYPE,
  PT_C POINT_TYPE,

  STATIC FUNCTION GETAREA(LAB in VARCHAR2)
    RETURN NUMBER
  AS LANGUAGE JAVA
  NAME 'Triangle.getArea(java.lang.String)return double'
)
/


CREATE TABLE TRIANGLES (
  LABEL VARCHAR2(25),
  TRIANGLE TRIANGLE_TYPE,
  CONSTRAINT TRIANGLE_PK
    PRIMARY KEY(LABEL)
);

INSERT INTO TRIANGLES
    VALUES('One',
    TRIANGLE_TYPE(POINT_TYPE(1,2),POINT_TYPE(5,6), POINT_TYPE(3,2)));

INSERT INTO TRIANGLES
  VALUES('Two',
    TRIANGLE_TYPE(POINT_TYPE(0,0),POINT_TYPE(0,4), POINT_TYPE(4,0)));

COMMIT;
