// Element.java
import java.sql.*;
import oracle.jdbc.pool.OracleDataSource;
import oracle.jdbc.driver.*;
import oracle.sql.*;
import java.math.BigDecimal;
import java.util.*;

public class Element implements SQLData {
  private static Connection conn = null;
  private static boolean typeMapSet = false;
  private static final String typeName="DAVID.ELEMENT_TYPE";
  private int    number;
  private String symbol;
  private String name;
  private float  weight;

  /* Constructors */
  public Element()
  {
  }
  
  private Element(int number, String symbol,
     String name, float weight)
  throws SQLException
  {
     // this.typeName = typeName;
     this.number = number;
     this.symbol = symbol;
     this.name = name;
     this.weight = weight;
   }

  // Interface implementation ...
   public String getSQLTypeName()
   {
     return typeName;
   }

   public void readSQL(SQLInput inStream, String typeName)
   throws SQLException
   {
    // typeName = typeName; already set
    number = inStream.readInt();
    symbol = inStream.readString();
    name   = inStream.readString();    
    weight = inStream.readFloat();
   }

   public void writeSQL(SQLOutput outStream)
   throws SQLException
   {
     outStream.writeInt(number);
     outStream.writeString(symbol);
     outStream.writeString(name);    
     outStream.writeFloat(weight);
   }

  // utility methods
  public static Connection getConnection()
  throws SQLException
  {
    if(conn==null)
    {
      OracleDataSource ods = new OracleDataSource();
      ods.setDriverType("thin");
      ods.setServerName("noizmaker");
      ods.setNetworkProtocol("tcp");
      ods.setDatabaseName("osiris");
      ods.setPortNumber(1521);
      ods.setUser("david");
	ods.setPassword("bigcat");
	conn = ods.getConnection();
    }
    setTypeMap();
    return conn;
  }

  private static void setTypeMap()
   throws SQLException
  {
    if(!typeMapSet)
    {
      Map typeMap = conn.getTypeMap();
      try
      {
         typeMap.put(typeName, Class.forName("Element"));
         typeMapSet = true;
      }
      catch (ClassNotFoundException e)
      {
        System.out.println("Caught: " + e);
      }
    }
  }
 
   
   // Access methods
      // Create new Element
  public static Element createElement(int number, String symbol,
    String name, float weight)
  throws SQLException
  {
    getConnection();
    Element e = null;
    if((e = getElement(number)) == null)
    {
      e = new Element(number, symbol, name, weight);
      PreparedStatement ps = conn.prepareStatement(
        "INSERT INTO ELEMENTS VALUES (?)");
      ps.setObject(1, e);
      ps.executeQuery();
      ps.close();
    }
    return e;
  }

  public static Element getElement(int number)
  throws SQLException
  {
    getConnection();
    PreparedStatement ps = conn.prepareStatement(
         "SELECT * FROM ELEMENTS E " +
         "WHERE E.ELEMENT.ATOMIC_NUMBER = ?");
    ps.setInt(1, number);
    return executePSQuery(ps);
  }

  public static Element getElement(String symbol)
  throws SQLException
  {
    getConnection();
    PreparedStatement ps = conn.prepareStatement(
         "SELECT * FROM ELEMENTS E " +
         "WHERE E.ELEMENT.SYMBOL = ?");
    ps.setString(1, symbol);
    return executePSQuery(ps);
  }
  
  private static Element executePSQuery(PreparedStatement ps)
  throws SQLException
  {
    Element e = null;
    ResultSet rs = ps.executeQuery();
    while(rs.next())
    {
      e = (Element) rs.getObject(1);
    }
    rs.close();
    ps.close();
    return e;
  }

  int getNumber()
  {
    return number; 
  }

  String getSymbol()
  {
    return symbol;
  }

  String getName()
  {
    return name;
  }
   
  float getWeight()
  {
    return weight;
  }

  int getNeutrons()
  {
    return Math.round(weight) - number;
  }
}

