import java.sql.*;
import oracle.sql.*;
import oracle.jdbc.*;
import java.math.BigDecimal; 
import oracle.jdbc.pool.OracleDataSource;

public class ORADataElement implements ORAData, ORADataFactory {

  static final ORADataElement _oraDataElementFactory =
    new ORADataElement();
    
  int number;
  String symbol;
  String name;
  float weight;

  public ORADataElement ()
  {
  }

  public ORADataElement (int number, String symbol,
    String name, float weight)
  {
    this.number = number;
    this.symbol = symbol;
    this.name = name;
    this.weight = weight;
  }

  public static ORADataFactory getFactory()
  {
     return _oraDataElementFactory;
  }

 // interface implementation - ORAData
  public Datum toDatum(Connection conn) throws SQLException
  {
    StructDescriptor sd =
      StructDescriptor.createDescriptor("ELEMENT_TYPE", conn);

        Object [] attributes = {
        new NUMBER(number),
        new CHAR(symbol, CHAR.DEFAULT_CHARSET),
        new CHAR(name, CHAR.DEFAULT_CHARSET),
        new NUMBER(weight)};
      
      return new STRUCT(sd, conn, attributes);
  } 

  // interface implementation - ORADataFactory
  public ORAData create(Datum d, int sqlType) throws SQLException
  {
    if(d==null)
    {
      return null;
    }
    Object [] attributes = ((STRUCT) d).getOracleAttributes();
    System.out.println("Datum is" + d.getClass().getName());
    return new ORADataElement(
       ((NUMBER) attributes[0]).intValue(),
       ((CHAR)attributes[1]).toString(),
       ((CHAR)attributes[2]).toString(),
       ((NUMBER) attributes[3]).floatValue());
   }
   
   public static void main(String [] args)
   {
     try
     {
        OracleDataSource ods = new OracleDataSource();
        ods.setDriverType("thin");
        ods.setServerName("noizmaker");
        ods.setNetworkProtocol("tcp");
        ods.setDatabaseName("osiris");
        ods.setPortNumber(1521);
        ods.setUser("david");
	      ods.setPassword("bigcat");
	      Connection conn = ods.getConnection();

        ORADataElement ee = new ORADataElement(16,"S","Sulfur", 32.06F);
        OraclePreparedStatement ps = (OraclePreparedStatement)
          conn.prepareStatement("INSERT INTO ELEMENTS VALUES(?)");
        ps.setORAData(1, ee); 
        ps.execute();
        ps.close();
        
        Statement stmt = conn.createStatement();
        OracleResultSet rs = (OracleResultSet) stmt.executeQuery(
          "SELECT * FROM ELEMENTS");
        while(rs.next())
        {
          ORADataElement e = (ORADataElement) rs.getORAData(1,
              ORADataElement.getFactory());
          System.out.println("Name:" + e.name); 
         }
         
       }
       catch (SQLException e)
       {
         System.out.println("Caught: " + e);
       }
   }
}
