// scrolling example
import java.sql.*;
import java.util.*;

public class ScrollWindow {

  public static final int ROWS_PER_PAGE = 20;
  
  public static final int SCROLLUP   = -1;
  public static final int SCROLLDOWN =  1;
  
  public static final int REFRESH =   0;
  public static final int PAGEUP =    1;
  public static final int PAGEDOWN =  2;
  public static final int HOME =      3;
  public static final int END =       4;
  public static final int UP =        5;
  public static final int DOWN =      6;

  // Instance variables
  int rowCount;
  int pages;
  int columnCount;

  Vector v = new Vector();
  ResultSet rs;
    
  ScrollWindow(ResultSet rs)
  throws SQLException
  {
      this.rs = rs;
      rs.last();
      rowCount = rs.getRow();
      pages = rowCount/ROWS_PER_PAGE;
      if(pages * ROWS_PER_PAGE < rowCount)
      {
        pages++;
      }
      rs.first();
      ResultSetMetaData rsmd = rs.getMetaData();
      columnCount = rsmd.getColumnCount();
 }
  
  public Vector getDataVector(int move)
  throws SQLException
  {
    // Only REFRESH is valid if too few results
    if(pages==1) 
    {
       move = REFRESH;
    }
    
    int startRow = rs.getRow();
    switch(move)
    {
      case PAGEUP:   if(startRow >= ROWS_PER_PAGE)
                     {
                       rs.relative(-ROWS_PER_PAGE);
                     }
                     break;
      case PAGEDOWN: if(startRow < (rowCount-(ROWS_PER_PAGE*2)))
                     {
                       rs.relative(ROWS_PER_PAGE);
                     }
                     else
                     {
                       rs.absolute(-ROWS_PER_PAGE);
                     }
                     break;
      case HOME:     rs.first();
                     break;
      case END:      rs.absolute(-ROWS_PER_PAGE);
                     break;
      case UP:       if(startRow > 1)
                     {
                       rs.previous();
                     }
                     break;
      
      case DOWN:     if(startRow <= rowCount - ROWS_PER_PAGE)
                     {
                       rs.next();
                     }
                     break;
      case REFRESH:  startRow = -1;
                     break;
    }

    switch(rs.getRow() - startRow)
    {
      case  0: break; // no move, no action  
      case -1: scrollVector(SCROLLUP);
               break;
      case  1: scrollVector(SCROLLDOWN);
               break;
      default: fillVector();
               break;
    }
    return v;
  }
  
  public void scrollVector(int scrollDirection)
  throws SQLException
  {
    System.out.println("scrolling: " + scrollDirection);
    String [] sArray;
    if((sArray = readRow())!=null);
    {
      switch(scrollDirection)
      {
        case SCROLLUP:   if((sArray = readRow())!=null)
                         {
                           v.removeElementAt(v.size()-1);
                           v.insertElementAt(sArray, 0);
                         }
                         break;
        case SCROLLDOWN: int startRow = rs.getRow();
                         rs.relative(ROWS_PER_PAGE - 1);
                         if((sArray = readRow())!=null)
                         {
                           v.removeElementAt(0);
                           v.add(sArray);
                         }
                         rs.absolute(startRow);
                         break;
      }
    }
  }
  
  public void fillVector()
  throws SQLException  
  {
    v.clear();
    int startRow = rs.getRow(); 
    rs.previous();
    
    for(int i=0; i < ROWS_PER_PAGE && rs.next(); i++)
    {
      v.add(readRow());
    }
    rs.absolute(startRow);    
  }

  public String [] readRow()
  throws SQLException
  {
    String [] sArray = new String [columnCount];

    for(int i=1; i<=columnCount; i++)
    {
      sArray[i - 1] = rs.getString(i);
    }
    return sArray;
  }

  
  /* *** TEST ROUTINES */
  
  public static void main(String [] vars)
  {
   Connection conn;
   String sql;
   try
   {
	    DriverManager.registerDriver(new oracle.jdbc.OracleDriver());
	    conn = DriverManager.getConnection(
                          "jdbc:oracle:thin:@noizmaker:1521:osiris",
                          "david", "bigcat");

      Statement stmt = conn.createStatement(
                            ResultSet.TYPE_SCROLL_INSENSITIVE,
                            ResultSet.CONCUR_READ_ONLY); 

      testData(stmt);
      ResultSet rs = stmt.executeQuery(
                 "SELECT NAME, DESCRIPTION, MODEL, PRICE " +
                 "FROM TEST_SCROLL");

      ScrollWindow sw = new ScrollWindow(rs);
      testMove(sw);
   }
   catch (SQLException e)
   {
     System.out.println("Caught: " + e);
   }
  }

  public static void testData(Statement stmt)
  throws SQLException
  {
     try
     {
       stmt.executeUpdate("DROP TABLE TEST_SCROLL");
     }
     catch (SQLException e)
     {
     }
     
     stmt.executeUpdate("CREATE TABLE TEST_SCROLL( " +
                          "NAME        VARCHAR2(20), " +
                          "DESCRIPTION VARCHAR2(20), " +
                          "MODEL       VARCHAR2(20), " +
                          "PRICE       NUMBER)");
     for(int i=0; i < 100; i++)
     {
       String sql = "INSERT INTO TEST_SCROLL VALUES("+
       
                    "'aaa'," +
                    "'bbb'," +
                    "'ccc'," +
                     i + ")";
       stmt.executeUpdate(sql);
     }

  }
  
  public static void testMove(ScrollWindow sw)
  throws SQLException
  {
    Vector v;
 
    System.out.println("REFRESH");
    v = sw.getDataVector(REFRESH);
    testPrint(v);
    
    System.out.println("END - 1");
    v = sw.getDataVector(END);
    testPrint(v);
    
    System.out.println("UP - 2");
    v = sw.getDataVector(UP);
    testPrint(v);
    
    System.out.println("UP - 3");
    v = sw.getDataVector(UP);
    testPrint(v);
    
    System.out.println("PAGEDOWN - 4");
    v = sw.getDataVector(PAGEDOWN);
    testPrint(v);

    System.out.println("DOWN - 5");
    v = sw.getDataVector(DOWN);
    testPrint(v);

    System.out.println("UP - 6");
    v = sw.getDataVector(UP);
    testPrint(v);

    System.out.println("DOWN - 7");
    v = sw.getDataVector(DOWN);
    testPrint(v);
    
    System.out.println("DOWN - 7a");
    v = sw.getDataVector(DOWN);
    testPrint(v);

    System.out.println("UP - 7b");
    v = sw.getDataVector(UP);
    testPrint(v);

    System.out.println("DOWN - 7c");
    v = sw.getDataVector(DOWN);
    testPrint(v);
    
    System.out.println("PAGEDOWN - 8");
    v = sw.getDataVector(PAGEDOWN);
    testPrint(v);
    
    System.out.println("UP - 9");
    v = sw.getDataVector(UP);
    testPrint(v);

    System.out.println("PAGEDOWN - 10");
    v = sw.getDataVector(PAGEDOWN);
    testPrint(v);

    System.out.println("HOME - 11");
    v = sw.getDataVector(HOME);
    testPrint(v);
    
    System.out.println("UP - 12");
    v = sw.getDataVector(UP);
    testPrint(v);
    
    System.out.println("DOWN - 13");
    v = sw.getDataVector(DOWN);
    testPrint(v);
    
    System.out.println("PAGEDOWN - 14");
    v = sw.getDataVector(PAGEDOWN);
    testPrint(v);

  }
  public static void testPrint(Vector testV)
  {
    String [] sA;

    System.out.println("====================");
    for(int i = 0; i < testV.size(); i++)
    {
      sA = (String [])testV.elementAt(i);
      System.out.println("[" + i + "] " +
          sA[0] + "  " +
          sA[1] + "  " +
          sA[2] + "  " +
          sA[3]);
    }
  }
}
