// StructOR - Oracle object relational features with Struct
import java.sql.*;
import oracle.jdbc.pool.OracleDataSource;
import oracle.jdbc.driver.*;
import oracle.sql.*;
import java.math.BigDecimal;

public class StructOR {

  public static void main(String [] vars)
  {
    Connection conn;
    try
    {
	    OracleDataSource ods = new OracleDataSource();
	    ods.setDriverType("thin");
	    ods.setServerName("noizmaker");
	    ods.setNetworkProtocol("tcp");
	    ods.setDatabaseName("osiris");
	    ods.setPortNumber(1521);
	    ods.setUser("david");
	    ods.setPassword("bigcat");
	    conn = ods.getConnection();

      // Retrieving an object with STRUCT
      Statement stmt = conn.createStatement(
         ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE);
      ResultSet rs = stmt.executeQuery(
        "SELECT PERIOD, COLUMN_GROUP, ELEMENT FROM PERIODIC_TABLE");

      while(rs.next())
      {
        int period = rs.getInt(1);
        int group = rs.getInt(2);
        System.out.println("period: " + period + "  group: " + group);
        STRUCT elementStruct = (STRUCT) rs.getObject(3);

        Object elementAttr[] = elementStruct.getAttributes();
        int atomic_number = ((BigDecimal)elementAttr[0]).intValue();
        String symbol =  (String) elementAttr[1];
        String name =    (String) elementAttr[2];
        float mass =    ((BigDecimal) elementAttr[3]).floatValue();

        System.out.println ("atomic number: " + atomic_number);
        System.out.println ("atomic symbol: " + (String) elementAttr[1]);
        System.out.println ("         name: " + name);    
        System.out.println ("         mass: " + mass);
      }

      // Obtaining information about object's attributes
      String elemTypeName = "ELEMENT_TYPE";
      StructDescriptor elemDesc = 
        StructDescriptor.createDescriptor(elemTypeName, conn);
      ResultSetMetaData elemMetaData = elemDesc.getMetaData();
      int columns = elemMetaData.getColumnCount();
      System.out.println("Number of attributes: " + columns);
      System.out.println("Attributes: ");	
      for(int i = 1; i <= columns; i++)
      {
        System.out.print(elemMetaData.getColumnName(i)+"    ");
        System.out.println(elemMetaData.getColumnTypeName(i));
      }

      // Creating new object with STRUCT() constructor
      Object [] elemAttr = new Object [4];
      elemAttr[0] = new BigDecimal(4);    // Atomic number
      elemAttr[1] = "Be";                 // symbol
      elemAttr[2] = "Beryllium";          // name
      elemAttr[3] = new BigDecimal(9.012);// mass

      // Inserting with prepared statement
      STRUCT elemStruct = new STRUCT(elemDesc, conn, elemAttr);
      PreparedStatement ps = conn.prepareStatement(
        "INSERT INTO PERIODIC_TABLE VALUES(?, ?, ?)");
       ps.setInt(1, 2); // period
       ps.setInt(2, 2); // group
       ps.setObject(3, elemStruct);
       int rows = ps.executeUpdate();
       System.out.println("Updated: " + rows);
       
       // Inserting with updateable result set
       elemAttr[0] = new BigDecimal(5);   // Atomic number
       elemAttr[1] = "B";                 // symbol
       elemAttr[2] = "Boron";             // name
       elemAttr[3] = new BigDecimal(10.81);// mass
       elemStruct = new STRUCT(elemDesc, conn, elemAttr);
       rs.moveToInsertRow();
       rs.updateInt(1,2);  // period
       rs.updateInt(2,13); // group
       rs.updateObject(3, elemStruct);
       // rs.insertRow();
    }
    catch(SQLException e)
    {
      System.out.println("Caught: " + e);
    }
  }
}

