package mypackage8.impl;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.FinderException;
import javax.ejb.CreateException;
import java.rmi.RemoteException;
import javax.naming.InitialContext;
import java.sql.*;

public class UserInfoBean implements EntityBean 
{
  public EntityContext entityContext;
  String email;
  String username;
  String password;
  java.util.Date memberSince;
  Connection conn;
  
  public String ejbCreate(String email,
      String username, String password)
  {
    System.out.println("--- ejbCreate ---");
    this.email = email;
    this.username = username;
    this.password = password;
    this.memberSince = new java.sql.Date((new java.util.Date()).getTime());
    return email;
  }

  public void ejbPostCreate(String email,
      String username, String password)
  throws CreateException
  {
    System.out.println("--- ejbPostCreate ---");
    getConnection();
    try
    {
      String sql = "INSERT INTO USER_INFO " +
                   "(EMAIL, USERNAME, PASSWORD, MEMBER_SINCE) " +
                   "VALUES(?,?,?,?)";
      PreparedStatement ps = conn.prepareStatement(sql);
      ps.setString(1, email);
      ps.setString(2, username);
      ps.setString(3, password);
      ps.setDate(4, new java.sql.Date(memberSince.getTime()));    
      ps.executeUpdate();
      ps.close();
    }
    catch(SQLException e)
    {
        throw new CreateException("Unable to create new record.");   
    }
  }

  public String ejbFindByPrimaryKey(String email)
  throws FinderException
  {
    System.out.println("--- ejbFindByPrimaryKey ---");
    if (email == null || email.equals(""))
    {
      throw new FinderException("Primary key cannot be null");
    }
    boolean success = false;
    try
    {
      String sql = "SELECT EMAIL FROM USER_INFO " +
                 "WHERE EMAIL = ?";
      getConnection();
      PreparedStatement ps = conn.prepareStatement(sql);
      ps.setString(1, email);
      ps.executeQuery();
      ResultSet rs = ps.getResultSet();
      success = rs.next();
      ps.close();
    }
    catch(SQLException e)
    {
      System.out.println("ejbFindByPrimaryKey() caught: " + e);
    }
    if(!success)
    {
      throw new FinderException("PK email not found");
    }
    disconnect();
    return email;
  }

  public void ejbActivate()
  {
    System.out.println("--- ejbActivate ---");
    getConnection();
  }

  public void ejbLoad()
  {
    System.out.println("--- ejbLoad ---");
    try
    {
      this.email = (String) entityContext.getPrimaryKey();
      String sql = "SELECT USERNAME, PASSWORD, MEMBER_SINCE " +
                   "FROM USER_INFO WHERE EMAIL=?";
      PreparedStatement ps = conn.prepareStatement(sql);
      ps.setString(1, email);
      ResultSet rs = ps.executeQuery();
      if(rs.next())
      {
        username = rs.getString(1);
        password = rs.getString(2);
        memberSince = rs.getDate(3);
      }
      ps.close();
    }
    catch(SQLException e)
    {
      System.out.println("ejbLoad() caught: " + e);
    }
  }

  public void ejbStore()
  throws RemoteException
  {
    System.out.println("--- ejbStore ---");  
   boolean success = false;
   try
   {
    String sql = "UPDATE USER_INFO SET " +
                 "USERNAME = ?, " +
                 "PASSWORD = ?, " +
                 "WHERE EMAIL = ? ";
                
    PreparedStatement ps = conn.prepareStatement(sql);
    ps.setString(1, username);
    ps.setString(2, password);
    ps.setString(3, email);
    success = (ps.executeUpdate() == 1);
    ps.close();
    }
    catch (SQLException e)
    {
       System.out.println("ejbStore() caught: " + e);
    }
    if(!success)
    {
      throw new RemoteException("Failed to update");
    }
  }

  public void ejbPassivate()
  {
    System.out.println("--- ejbPassivate ---");
    disconnect();
  }

  public void ejbRemove()
  throws RemoteException  
  {
    System.out.println("--- ejbRemove ---");
    boolean success = false;
    try
    {
      String sql = "DELETE FROM USER_INFO WHERE " +
                    "EMAIL = ?";
      PreparedStatement ps = conn.prepareStatement(sql);
      ps.setString(1, this.email);
      success = (ps.executeUpdate() == 1);
      ps.close();
      disconnect();
    }
    catch(SQLException e)
    {
      System.out.println("ejbRemove() caught: " +e);
    }
    if(!success)
    {
      throw new RemoteException("Failed to delete record");
    }
  }


  public void setEntityContext(EntityContext ctx)
  {
    System.out.println("--- setEntityContext ---");
    this.entityContext = ctx;
  }

  public void unsetEntityContext()
  {
    System.out.println("--- unsetEntityContext ---");
    this.entityContext = null;
  }

  private void getConnection()
  {
    System.out.println("*** getConnection() ***");
    if(conn == null)
    {
      try
      {
        InitialContext ic=new InitialContext();
        javax.sql.DataSource ds=(javax.sql.DataSource)(ic.lookup("jdbc/OsirisDS"));
        conn= ds.getConnection();
      }
      catch(Exception e)
      {
        System.out.println("getConnection() caught: " + e);
      }
    }
  }

  private void disconnect()
  {
    System.out.println("*** disconnect() ***");
    if(conn!=null)
    {
      try
      {
        conn.close();
        conn = null;
      }
      catch(SQLException e)
      {
        System.out.println("disconnect() caught: " + e);
      }
    }
  }


  public String getUsername()
  {
    return username;
  }

  public String getEmail()
  {
    return email;
  }

  public String getPassword()
  {
    return password;
  }
  
  public java.util.Date getMemberSince()
  {
    return memberSince;
  }
  public void setUsername(String username)
  {
    this.username = username;
  }
  public void setPassword(String password)
  {
    this.password = password;
  }
  public String getHello()
  {
    return "Hello";
  }
}